import math, sets
from sudoku import InputReader, Report

class Skeleton(object):
	def __init__(self, size=3, type='S', grid=None, array=None, blank=False, file=None, string='', id=None, cursor=None):
		"types are: K = kalidoscope, Q = square, P = pinwheel, S = S"
		self.__pattern = []
		self.__symmetry = self.__allocateSymmetry(type)
		self.report = Report(self) #object which controlls reporting

		if grid:
			size = grid.size
		elif array != None:  # must use != None here!
			size = int(math.sqrt(len(array)))
		# following alternative sets the size when data comes from a file or string
		elif not blank and file <> "BLANK":
			self.cursor = cursor
			if id: string = self.dataFromDb(id)
			size = self.setReader(file=file, string=string, size=size) or size
		self.__len = size * size
		self.zeroValues()

	def zeroValues(self):
		self.__pattern = [[False for c in range(self.__len)] for r in range(self.__len)]
	
	def listValues(self): return [v for row in self.__pattern for v in row]

	def count(self, silent=True):
		"counts the numbers of set and unset cells"
		unset = self.listValues().count(False)
		set = len(self.listValues()) - unset
		if not silent and self.report and self.report.do(1):
			self.report.message(("%d cells have values, %d are unset" % 
									(set, unset)),1)
		return set
	
	def dataFromDb(self, id):
		self.Db.execute('select data from skeleton where id = %s', id)
		return self.Db.fetchone()[0]
			
	def saveDb(self, symmetry='N'):
		self.Db.execute("""insert into skeleton (size, data, filled, symmetry) values (%s, %s, %s, %s)""", (self.size**4, self.writeIt(), self.count(), self.symmetry))
		self.Db.execute("""select last_insert_id()""")
		id = int(self.Db.fetchone()[0])
		return id

	def __allocateSymmetry(self, type):
		if type == 'Q': return self.__qSymmetry
		elif type == 'P': return self.__pSymmetry
		elif type == 'K': return self.__kSymmetry
		elif type == 'B': return self.__bSymmetry
		elif type == 'D': return self.__dSymmetry
		elif type == 'N': return self.__nSymmetry
		else: return self.__sSymmetry

	def __unallocateSymmetry(self):
		if self.__symmetry == self.__pSymmetry: return 'P'
		elif self.__symmetry == self.__qSymmetry: return 'Q'
		elif self.__symmetry == self.__sSymmetry: return 'S'
		elif self.__symmetry == self.__kSymmetry: return 'K'
		elif self.__symmetry == self.__bSymmetry: return 'B'
		elif self.__symmetry == self.__dSymmetry: return 'D'
		elif self.__symmetry == self.__nSymmetry: return 'N'
		else: return None
	
	def __getSize(self):
		return int(math.sqrt(self.__len))
	size = property(__getSize)

	def __setSymmetry(self, type):
		"set symmetry to the specified type"
		current = self.__unallocateSymmetry() 
		self.__symmetry = self.__allocateSymmetry(type)
	symmetry = property(__unallocateSymmetry, __setSymmetry)

	def getValue(self, r, c):
		return self.__pattern[r][c]
	
	def __getRc(self, row=-1, col=-1, r=-1, c=-1, n=-1):
		"specify either row and col (numbered from 1), r and c (numberered from 0) or n"
		if row < 0:
			if r < 0:
				r = n / self.__len
				c = n % self.__len
		else:
			r = row - 1
			c = col - 1
		return (r, c)

	def getCells(self, row=-1, col=-1, r=-1, c=-1, n=-1):
		return self.set(row=row, col=col, r=r, c=c, n=n, dontSet=True)

	def set(self, row=-1, col=-1, value=True, r=-1, c=-1, n=-1, dontSet=False):
		"""sets the value of the cell and others implied by the prevailing symmetry
		
		specify either row and col (numbered from 1), 
		r and c (numberered from 0) or 
		n"""
		cells = self.__symmetry(self.__getRc(row, col, r, c, n))
		if not dontSet:
			for cell in cells: self.__pattern[cell[0]][cell[1]] = value
		cells = self.__reformat(row, r, n, cells)
		return cells

	def __reformat(self, row, r, n, cells):
		"converts the coordinates of cells to the required format"
		if row < 0:
			if r < 0: # return n
				return tuple([r*self.__len + c for (r, c) in cells])
			else:     # return (r,c)    - ie 0 based coordinates
				return cells
		else:         # return(row,col) - ie 1 based coordinates
			return tuple([(r+1, c+1) for (r, c) in cells])

	def __str__(self):
		return "\n".join([' '.join([(self.__pattern[r][c] and '*' or ' ') for c in range(self.__len)]) for r in range(self.__len)])
		
	def __sSymmetry(self, cell):
		(r, c) = cell
		cells = sets.Set()
		cells.add(cell)
		cells.add((self.__len-r-1, self.__len-c-1))
		return tuple(cells)

	def __bSymmetry(self, cell):
		(r, c) = cell
		cells = sets.Set()
		cells.add(cell)
		cells.add((r, self.__len-c-1))
		return tuple(cells)

	def __dSymmetry(self, cell):
		(r, c) = cell
		cells = sets.Set()
		cells.add(cell)
		cells.add((self.__len-c-1,self.__len-r-1))
		return tuple(cells)

	def __pSymmetry(self, cell):
		(r, c) = cell
		cells = sets.Set()
		cells.add(cell)
		cells.add((self.__len-r-1, self.__len-c-1))
		cells.add((c, self.__len-r-1))
		cells.add((self.__len-c-1, r))
		return tuple(cells)

	def __qSymmetry(self, cell):
		(r, c) = cell
		cells = sets.Set()
		cells.add(cell)
		cells.add((self.__len-r-1, c))
		cells.add((r, self.__len-c-1))
		cells.add((self.__len-r-1, self.__len-c-1))
		return tuple(cells)

	def __kSymmetry(self, cell):
		(r, c) = cell
		cells = sets.Set()
		cells.add(cell)
		cells.add((self.__len-r-1, c))
		cells.add((r, self.__len-c-1))
		cells.add((self.__len-r-1, self.__len-c-1))
		cells.add((c, r))
		cells.add((c, self.__len-r-1))
		cells.add((self.__len-c-1, r))
		cells.add((self.__len-c-1, self.__len-r-1))
		return tuple(cells)
	

	def __nSymmetry(self, cell):
		return (cell,)
	
	def setReader(self, file=None, string='', size=3):
		if file or string: # in these cases set the size from the data
			if file: self.reader = InputReader(input = file)
			else: self.reader = InputReader(string = string)
			return self.reader.size
		else: # in this case use the size which was specified in the Grid initializer
			self.reader = InputReader(size = size)
			return None

	def writeIt(self):
	
		"returns a print string for the grid suitable for output to a file"
		return ("['"+"', '".join([('%s'*self.size*self.size % 
						tuple([self.getValue(i,j) and '1' or '0' 
						for j in range(self.size*self.size)])) 
						for i in range(self.size*self.size)])+"']\n")

	def read(self):
		"reads values from the terminal or file and sets the starting values of the grid"
		dataline = self.reader.dataLine()
		for i in range(self.__len):
			data = dataline.next()
			for j in range(self.__len):
				self.__pattern[i][j] = eval(data[j]) and True or False
		self.reader.close()


