#!/usr/bin/env python
import sys, wx, wx.grid, wx.html
import sudoku, sudokuH, xmlReader
import wxWin 
from sudokuTable import SudokuTable, SolutionStep

VERSION = '0.1'

#****************************** class SudikoWindow *********************************

class SudokuWindow(wx.Frame):

	def __init__(self, *args, **kwds):
		self.app = kwds.get("app",None)
		grid = kwds.get("grid",None)
		wx.Frame.__init__(self, None, -1, "sudoku", size=(325,680))
		self.box = wx.BoxSizer(wx.VERTICAL)
		self.table = self.createTable(grid=grid)
		self.box.Add(self.table)
		s = self.table.grid.size**2 * 32 + 20
		self.table.SetSizeHints(s,s)
		self.box.SetMinSize((s,640))
		self.messageArea = wx.BoxSizer(wx.HORIZONTAL)
		self.message = wx.TextCtrl(self, -1, size=(500,300), style=wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_WORDWRAP)
		self.messageArea.Add(self.message)
		self.prevButton = wx.Button(self, -1, "prev")
		self.messageArea.Add(self.prevButton)
		self.nextButton = wx.Button(self, -1, "next")
		self.messageArea.Add(self.nextButton)
		self.box.Add(self.messageArea)
		self.SetSizerAndFit(self.box)
		self.mPrint = 0
		self.mSteps = 0
		self.itemsToReenable = []
		self.mPossibles = None # may be overridden
		self.helpControl = None
		self.aboutDlg = None
		
		wx.EVT_BUTTON(self, self.prevButton.GetId(), self.prev)
		wx.EVT_BUTTON(self, self.nextButton.GetId(), self.next)
		self.nextButton.Enable(False)
		self.prevButton.Enable(False)
		self.listReporter = None
		self.gridMenu = None
		self.highlightItem = 0
		self.moreHighlightItem = 0
		self.restoreItem = 0
		self.selectItem = 0
		self.savedReport = None
		self.populate()
		self.printer = None
		grid.report = GuiReporter(self)
		print type(grid)

	def createTable(self, grid):
		return SudokuTable(self, grid=grid)

	def populate(self):
		mb = wx.MenuBar()
		fm = wx.Menu()
		om = wx.Menu()
		self.mOpen = om.Append(-1, "open &text file...\tCTRL+E").GetId() #self.open
		self.mOpenCsv = om.Append(-1, "open &csv file...\tCTRL+C").GetId() #self.open
		self.mOpenXml = om.Append(-1, "open &xml file...\tCTRL+L").GetId() #self.open
		fm.AppendMenu(-1, "&open file\tCTRL+O", om)
		sm = wx.Menu()
		self.mSave = sm.Append(-1, "save &text file as...\tCTRL+T").GetId() #self.save
		self.mSaveCsv = sm.Append(-1, "save &csv file as...\tCTRL+V").GetId() #self.save
		self.mSaveXml = sm.Append(-1, "save &xml file as...\tCTRL+M").GetId() #self.save
		fm.AppendMenu(-1, "&save file", sm)
		
		fm.AppendSeparator()
		self.mPrint = fm.Append(-1, "&print...\tCTRL+P").GetId()
		fm.AppendSeparator()
		mExit = fm.Append(wx.ID_EXIT, "e&xit\tCTRL+X").GetId() #self.exit
		mb.Append(fm, "&file")

		self.gridMenu = wx.Menu()
		mSolve = self.gridMenu.Append(-1, "sol&ve\tCTRL+V").GetId() #self.solve
		self.mSteps = self.gridMenu.AppendCheckItem(-1, "s&how solution steps\tCTRL+H").GetId() #self.hsolve
		self.mHighlight = self.highlightItem = self.gridMenu.AppendCheckItem(-1, "highlight &redundants\tCTRL+R").GetId() #self.highlightRedundants
		self.mCreate = self.gridMenu.Append(-1, "&create puzzle").GetId()
		self.mAnalyze = self.gridMenu.Append(-1, "&analyze puzzle").GetId()
		self.gridMenu.AppendSeparator()
		self.mLeftNothing = self.gridMenu.AppendCheckItem(-1, "left click does nothing").GetId() # self.clearRedundant
		self.gridMenu.Check(self.mLeftNothing, True)
		self.mClearRedundant = self.gridMenu.AppendCheckItem(-1, "left click to clear redundant value").GetId() # self.clearRedundant
		self.gridMenu.Enable(self.mClearRedundant, False)
		self.mClearClicked = self.gridMenu.AppendCheckItem(-1, "left click to clear any value").GetId() # self.clearClicked
		self.mChangePossibles = self.gridMenu.AppendCheckItem(-1, "left click to edit &candidates list").GetId()
		self.gridMenu.Enable(self.mChangePossibles, False)
		self.gridMenu.AppendSeparator()
		self.mRightNothing = self.gridMenu.AppendCheckItem(-1, "right click does nothing").GetId() # self.restoreValue
		self.gridMenu.Check(self.mRightNothing, True)
		self.mRestore = self.gridMenu.AppendCheckItem(-1, "right click to restore original value").GetId() # self.restoreValue
		self.mSelect = self.selectItem = self.gridMenu.AppendCheckItem(-1, "right click to select a value").GetId() # self.selectValue
		self.mAny = self.anyItem = self.gridMenu.AppendCheckItem(-1, "right click to select any value").GetId() # self.selectValue
		self.gridMenu.AppendSeparator()
		mCount = self.gridMenu.Append(-1, "cou&nt values\tCTRL+N").GetId() #self.countValues
		mClear = self.gridMenu.Append(-1, "clear &all").GetId() #self.clearAll
		mb.Append(self.gridMenu, "&grid")

		em = wx.Menu()
		mUndo = em.Append(-1,"&undo\tCTRL+Z").GetId() #self.undo
		mRedo = em.Append(-1,"&redo\tCTRL+R").GetId() #self.redo
		mb.Append(em, "&edit")

		self.settingsMenu = wx.Menu()
		mMax = self.settingsMenu.Append(-1, "set &maximum solutions\tCTRL+M").GetId()
		mFont = self.settingsMenu.Append(-1, "fo&nt\tCTRL+N").GetId() #self.font 
		mb.Append(self.settingsMenu, "&settings")
		helpMenu = wx.Menu()
		mAbout = helpMenu.Append(-1, '&about').GetId()
		mHelp = helpMenu.Append(-1, '&contents').GetId()
		mb.Append(helpMenu, "&help")

		self.SetMenuBar(mb)

		wx.EVT_MENU(self, self.mOpen, self.open)
		wx.EVT_MENU(self, self.mOpenCsv, self.open)
		wx.EVT_MENU(self, self.mOpenXml, self.open)
		wx.EVT_MENU(self, self.mSave, self.save)
		wx.EVT_MENU(self, self.mSaveCsv, self.save)
		wx.EVT_MENU(self, self.mSaveXml, self.save)
		wx.EVT_MENU(self, mExit, self.exit)
		wx.EVT_MENU(self, mSolve, self.solve)
		wx.EVT_MENU(self, self.mSteps, self.hsolve)
		wx.EVT_MENU(self, self.mCreate, self.create)
		wx.EVT_MENU(self, self.mAnalyze, self.analyze)
		wx.EVT_MENU(self, self.mHighlight, self.highlightRedundants)
		wx.EVT_MENU(self, self.mLeftNothing, self.directClicks)
		wx.EVT_MENU(self, self.mRightNothing, self.directClicks)
		wx.EVT_MENU(self, self.mClearRedundant, self.directClicks)
		wx.EVT_MENU(self, self.mClearClicked, self.directClicks)
		wx.EVT_MENU(self, self.mChangePossibles, self.directClicks)
		wx.EVT_MENU(self, self.mRestore, self.directClicks)
		wx.EVT_MENU(self, self.mSelect, self.directClicks)
		wx.EVT_MENU(self, self.mAny, self.directClicks)
		wx.EVT_MENU(self, mCount, self.countValues)
		wx.EVT_MENU(self, mClear, self.clearAll)
		wx.EVT_MENU(self, mUndo, self.undo)
		wx.EVT_MENU(self, mRedo, self.redo)
		wx.EVT_MENU(self, mFont, self.font)
		wx.EVT_MENU(self, mMax, self.setMaximumSolutions)
		wx.EVT_MENU(self, self.mPrint, self.printIt)
		wx.EVT_MENU(self, mAbout, self.about)
		wx.EVT_MENU(self, mHelp, self.help)
	
	def create(self, event):
		print "in create - grid size = %d" % self.table.grid.size
		newgrid = sudoku.createForwards(symmetry='N', size=self.table.grid.size)
		if newgrid: 
			self.table.grid.restoreValues(newgrid.listValues())
			self.table.newFile()
			self.table.readAllValues()
			self.table.undoBuffer.store()
			self.table.clickRestore = self.table.grid.listValues()
			self.table.grid.report.message("new grid created")
			self.countValues(None)
		else:
			self.table.grid.report.message("unlucky this time")
			
	def analyze(self, event):
		keep = self.table.grid.listValues()
		self.table.grid.hsolve()
		self.table.grid.report.message(str(self.table.grid.statistics))
		self.table.grid.restoreValues(keep)
		self.table.readAllValues()
	
	def about(self, event):
		if not self.aboutDlg: self.aboutDlg = AboutDlg(self)
		self.aboutDlg.ShowModal()

	def help(self, event):
		if not self.helpControl: 
			self.helpControl = wx.html.HtmlHelpController(wx.html.HF_CONTENTS|
				wx.html.HF_PRINT|wx.html.HF_SEARCH|wx.html.HF_TOOLBAR|
				wx.html.HF_BOOKMARKS)
			self.helpControl.SetTempDir("/var/tmp") #not portable!!
			self.helpControl.AddBook("help/help.hhp")
			self.helpControl.SetTitleFormat("Sudoku Help - %s")
		self.helpControl.DisplayContents()
			
	def printIt(self, event):
		if not self.printer: self.printer = Printer(self)
		self.printer.printit(self.table.grid)
	
	def exit(self, event):
		self.Close()
		self.app.ExitMainLoop() # this is needed if dbDialog is used!!
	
	def undo(self, event):
		self.table.undoBuffer.undo()

	def redo(self, event):
		self.table.undoBuffer.redo()

	def font(self, event):
		dialog = wx.FontDialog(self,wx.FontData())
		ok = dialog.ShowModal() == wx.ID_OK
		if not ok: return
		font = dialog.GetFontData().GetChosenFont()
		self.table.SetDefaultCellFont(font)

	def open(self, event):
		standard = "All files|*|"\
				"Puzzle files (*.puzzle, *.puz)|*.puzzle;*.puz|"\
				"Solution files (*.solution, *.solved, *.sol)|*.solution;*.solved;*.sol|"\
				"Data files (*.dat)|*.dat|"\
				"Skeleton files (*.skeleton, *.skel)|*.skeleton;*.skel"
	
		if event.GetId() == self.mOpen:
			wildcard = standard
		elif event.GetId() == self.mOpenCsv:
			wildcard = "csv files (*.csv)|*.csv|"+standard
		elif event.GetId() == self.mOpenXml:
			wildcard = "xml files (*.xml)|*.xml|"+standard
		dialog = wx.FileDialog(self,"file to open","","", wildcard, 
				wx.OPEN|wx.FILE_MUST_EXIST)
		ok = dialog.ShowModal() == wx.ID_OK
		name = dialog.GetPath()
		if not ok: return
		if event.GetId() == self.mOpenXml: self.table.open(xmlFile=name)
		elif event.GetId() == self.mOpenCsv: self.table.open(csvFile=name)
		else: self.table.open(file=name)
		self.SetStatusText("read data from file: %s" % name)
		self.SetTitle("sudoku - %s" % name)
		#self.SetStatusText(str(self.table.grid.count()),self.counter)

	def countValues(self, event):
		self.table.grid.count()		
	
	def setMaximumSolutions(self, event):
		(ok, max) = MaxDialog.getValues(self)
		if ok: self.table.maxSolutions = max
		
	def __resetMessages(self):
		self.prevButton.setEnabled(False)
		self.nextButton.setEnabled(False)
		self.table.grid.report.message("")

	def exit(self, event):
		self.Close()
		self.app.ExitMainLoop() # this is needed if dbDialog is used!!

	def save(self, event):
		standard = "All files|*|"\
		"Puzzle files (*.puzzle, *.puz)|*.puzzle;*.puz|"\
		"Solution files (*.solution, *.solved, *.sol)|*.solution;*.solved;*.sol|"\
		"Data files (*.dat)|*.dat|"\
		"Skeleton files (*.skeleton, *.skel)|*.skeleton;*.skel"

		if event.GetId() == self.mSave:
			wildcard = standard
		elif event.GetId() == self.mSaveCsv:
			wildcard = "csv files (*.csv)|*.csv|"+standard
		elif event.GetId() == self.mSaveXml:
			wildcard = "xml files (*.xml)|*.xml|"+standard
		dialog = wx.FileDialog(self,"file to save","","",wildcard,
				wx.SAVE|wx.OVERWRITE_PROMPT)
		ok = dialog.ShowModal() == wx.ID_OK
		if not ok: return
		name = dialog.GetPath()
		file = open(name, "w")
		if event.GetId() == self.mSave:
			file.write(self.table.grid.writeIt())
		elif event.GetId() == self.mSaveCsv:
			file.write(self.table.grid.writeCsv())
		elif event.GetId() == self.mSaveXml:
			file.write(self.writeXml())
		#self.SetStatusText("grid saved to file: %s" % name)
		self.SetTitle("sudoku - %s" % name)

	def writeXml(self):
		return str(xmlReader.XmlWriter(self.table.grid))

	def prev(self, event):
		self.table.visibleSolution -= 1
		if self.table.visibleSolution < 0:
			self.table.visibleSolution = self.table.grid.nsolutions - 1
		#self.__setupClearRedundant(False)
		self.table.showThisSolution()
	
	def next(self, event):
		self.table.visibleSolution += 1
		if self.table.visibleSolution >= self.table.grid.nsolutions:
			self.table.visibleSolution = 0
		#self.__setupClearRedundant(False)
		self.table.showThisSolution()

	def hsolve(self, event):
		if not self.gridMenu.IsChecked(self.mSteps):
			# turn solution steps off
			self.table.grid.report = self.invisibleReport 
			self.invisibleReport = None
			self.listReporter.clear()
			self.listReporter.hide()
			self.listReporter
			#reenable all menu items disabled when we started hsolve
			for i in self.itemsToReenable: i.Enable(True)
		else:
			# turn solution steps on
			self.invisibleReport = self.table.grid.report
			if not self.listReporter:
				self.listReporter = ListReporter(self)
			self.table.grid.report = self.listReporter
			#disable all menu items but the current one, possibles, print and exit
			self.itemsToReenable = []
			mb = self.GetMenuBar()
			for m in range(mb.GetMenuCount()):
				for mi in mb.GetMenu(m).GetMenuItems():
					if mi.GetId() in (self.mPrint, 
								wx.ID_EXIT, self.mSteps, self.mPossibles):
						continue
					if mi.IsEnabled():
						self.itemsToReenable.append(mi)
						mi.Enable(False)
			try:
				self.listReporter.clear()
				self.listReporter.linenum=0
				self.listReporter.message("start")
				self.table.grid.hsolve()	
				self.listReporter.finalize()
				self.listReporter.show()
				#were sending messages to the Gui Reporter which is invisible!!
			except sudoku.ImpossibleGridError, e:
				self.table.grid.report = self.invisibleReport
				self.invisibleReport = None
				self.table.grid.report.message(e.value)
				self.table.undoBuffer.restore()

	def solve(self, event):
		try:
			self.table.grid.solve(self.table.maxSolutions)	
		except RuntimeError:
			self.table.grid.report.message("too many solutions")
			#self.table.undoBuffer.restore()
			pass
		except sudoku.ImpossibleGridError, e:
			self.table.grid.report.message(e.value + "\nyou need to backtrack")
			self.table.undoBuffer.restore()
			return
		if self.table.grid.nsolutions > 1:
			self.prevButton.Enable(True)
			self.nextButton.Enable(True)
			self.next(None)
		else:
			self.table.visibleSolution = 0
			#self.__setupClearRedundant(False)
			self.table.showThisSolution()

	def highlightRedundants(self, event):
		if not self.gridMenu.IsChecked(self.mHighlight):
			self.gridMenu.Enable(self.mClearRedundant,False)
			self.table.removeHighlights()
		else:
			#turn redundant highlighting on
			self.gridMenu.Enable(self.mClearRedundant,True)
			self.table.highlightRedundants()

	def __doMenuChecks(self, on=None, off=None):
		for m in on: self.gridMenu.Check(m,True)
		for m in off: self.gridMenu.Check(m,False)
	
	def __leftOff(self):
		self.__doMenuChecks(on=(self.mLeftNothing,), 
				off=(self.mClearClicked,self.mClearRedundant,self.mChangePossibles))
		self.table.clearClicked = False
		self.table.clearRedundant = False
		self.table.changePossibles = False
		
	def __rightOff(self):
		self.__doMenuChecks(on=(self.mRightNothing,), 
				off=(self.mSelect,self.mRestore))
		self.table.selectValue = False
		self.table.restoreValue = False

	def directClicks(self, event):
		"emulates radio menu items and sets variables to be used by the table"
		if event.GetId() == self.mLeftNothing: 
			self.__leftOff()
		elif event.GetId() == self.mClearClicked: 
			if event.IsChecked():
				self.__doMenuChecks(on=(self.mClearClicked,), 
						off=(self.mLeftNothing,self.mClearRedundant,self.mChangePossibles))
				self.table.clearClicked = True
				self.table.clearRedundant = False
				self.table.changePossibles = False
			else: self.__leftOff()
		elif event.GetId() == self.mClearRedundant:
			if event.IsChecked():
				self.__doMenuChecks(on=(self.mClearRedundant,), 
						off=(self.mLeftNothing,self.mClearClicked,self.mChangePossibles))
				self.table.clearClicked = False
				self.table.clearRedundant = True
				self.table.changePossibles = False
			else: self.__leftOff()
		elif event.GetId() == self.mChangePossibles:
			if event.IsChecked():
				self.__doMenuChecks(on=(self.mChangePossibles,), 
						off=(self.mLeftNothing,self.mClearClicked,self.mClearRedundant))
				self.table.clearClicked = False
				self.table.clearRedundant = False
				self.table.changePossibles = True
			else: self.__leftOff()
		elif event.GetId() == self.mRightNothing: 
			self.__rightOff()
		elif event.GetId() == self.mSelect:
			if event.IsChecked():
				self.__doMenuChecks(on=(self.mSelect,), 
						off=(self.mRightNothing, self.mRestore, self.mAny))
				self.table.anyValue = False
				self.table.selectValue = True
				self.table.restoreValue = False
			else: self.__rightOff()
		elif event.GetId() == self.mAny:
			if event.IsChecked():
				self.__doMenuChecks(on=(self.mAny,),
						off=(self.mRightNothing, self.mRestore, self.mSelect))
				self.table.selectValue = True
				self.table.anyValue = True
				self.table.restoreValue = False
			else: self.__rightOff()
		elif event.GetId() == self.mRestore:
			if event.IsChecked():
				self.__doMenuChecks(on=(self.mRestore,),
						off=(self.mRightNothing, self.mSelect, self.mAny))
				self.table.selectValue = False
				self.table.restoreValue = True
			else: self.__rightOff()

	def clearAll(self, event):
		self.table.clearAllValues()
		self.table.undoBuffer.store()
		self.table.clickRestore = None
	
#****************************** class Printer *********************************

class Printer(wxWin.Printer):

	def OnPrintPage(self, n):
		z=self.grid.size
		self.drawBlank()
		
		(stepx, stepy) = self.lengthx/z/z, self.lengthy/z/z
		startx = self.marginx + self.lengthx/z/z/2
		starty = self.marginy + self.lengthy/z/z/2
	
		self.GetDC().SetFont(wx.Font(24, wx.FONTFAMILY_ROMAN, 
				wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
		for x in range(z*z):
			for y in range(z*z):
				v = self.grid.listValues()[y*z*z+x]
				self.centreText(v and str(v) or '', startx+x*stepx, starty+y*stepy) 
	
#****************************** class GuiReporter *********************************

class GuiReporter(object):

	def __init__(self, win):
		self.table = win.table
		self.canvas = win.message
		self.on = True
		self.printLevels = []
		self.verbose = False
		self.linenum = 0

	def __setVerbose(self, verbose):
		self.printLevels = verbose and range(4) or range(3)
	verbose = property(None, __setVerbose)

	def do(self, level):
		return self.on and level in self.printLevels

	def message(self, text, level=0, newline=True, linenum=False, cells=None):
		if not self.do(level): return
		if linenum: self.linenum += 1
		prefix = linenum and '[%d] ' % self.linenum or ''
		self.canvas.AppendText(prefix + text)
		self.newline(newline)
	
	def newline(self, newline):
		if newline: self.canvas.AppendText("\n")

	def setOn(self, on):
		self.on = on

	def display(self, level=0):
		if self.on and level in self.printLevels:
			self.table.readAllValues()
	
#*************************** class ListReporter *******************************

class ListReporter(GuiReporter):

	def __init__(self, win):
		self.win = win
		self.list = FormattedList(parent=win, size=[500,300])
		win.box.Add(self.list)
		GuiReporter.__init__(self, win)
		self.canvas = self
		self.verbose = True
		self.alternative = win.messageArea
		wx.EVT_LISTBOX(win, self.list.GetId(), self.clicked)

	def message(self, text, level=0, newline=True, linenum=False, cells=[]):
		if not self.do(level): return
		if linenum: self.linenum += 1
		prefix = linenum and '[%d] ' % self.linenum or ''
		self.canvas.append(prefix + text, cells)
	
	def clicked(self, event):
		item = event.GetEventObject().list[event.GetSelection()][1]
		item.restore(self.win.table.grid, item.status)
		addCells = False
		if item.cells:
			for (r,c) in item.cells:
				self.win.table.SelectBlock(r,c,r,c,addCells)
				addCells = True
		else: self.win.table.ClearSelection()
		self.win.table.readAllValues(True)
	
	def hide(self):
		self.win.table.select(on=False)
		self.list.Hide()
		self.win.box.Show(self.alternative)

	def show(self):
		self.win.box.Hide(self.alternative)
		self.list.Show()
		self.win.table.select(on=True)
	
	def clear(self): 
		self.list.Clear()
		self.linenum = 0
	
	def finalize(self):
		self.list.SetItemCount(len(self.list.list))

	def AppendText(self, text):
		self.append(text)

	def append(self, text, cells=()):
		SolutionStep(text=text, listbox=self, cells=cells)

#************************** class FormattedList *******************************

class FormattedList(wx.HtmlListBox):

	def __init__(self, parent, size=[500,300]):
		wx.HtmlListBox.__init__(self, parent=parent, id=-1, size=size)
		self.list=[]

	def Clear(self):
		wx.HtmlListBox.Clear(self)
		self.list=[]
		self.SetItemCount(0)

	def append(self, text, clientData):
		self.list.append((text, clientData))
	
	def OnGetItem(self, num):
		return self.list[num][0]

#************************** class OkCancel ********************************

class OkCancel(wx.BoxSizer):
	def __init__(self, parent, ok, cancel):
		wx.BoxSizer.__init__(self,wx.HORIZONTAL)
		self.Add((0,0),1)
		okButton = wx.Button(parent, -1, 'OK')
		self.Add(okButton,flag=wx.ALL,border=10)
		okButton.SetDefault()
		cancelButton = wx.Button(parent, -1, 'Cancel')
		self.Add(cancelButton,flag=wx.ALL,border=10)
		wx.EVT_BUTTON(parent,okButton.GetId(),ok)
		wx.EVT_BUTTON(parent,cancelButton.GetId(),cancel)

#************************** class MaxDialog *******************************

class MaxDialog(wx.Dialog):
	"""Dialog window for user to input maximum solutions to be calculated
	Only one dialog is meant to be instantiated.  
	Only the getValues class method should be called by other classes"""

	dialog = None	

	def __init__(self, parent):
		self.parent = parent
		wx.Dialog.__init__(self, None, -1, "maximum solutions",size=(300,220))
		self.max = self.parent.table.maxSolutions
		self.box = wx.BoxSizer(wx.VERTICAL)
		self.box.Add(wx.StaticText(self,-1,"maximum solutions:\n"
			"(enter a number greater than 1\nor click the "
			"'no maximum' button)"),border=10,flag=wx.TOP|wx.LEFT)
		self.maxField = wx.TextCtrl(self,-1,size=(40,25))
		self.box.Add(self.maxField,border=10,flag=wx.TOP|wx.LEFT)
		noButton = wx.Button(self, -1, 'no maximum')
		self.box.Add(noButton,flag=wx.ALL,border=10)
		
		strip = OkCancel(parent=self, ok=self.__ok, cancel=self.__cancel)
		self.box.Add(strip,proportion=1,flag=wx.ALL|wx.EXPAND,border=10)
		self.SetSizer(self.box)

		wx.EVT_BUTTON(self,noButton.GetId(),self.__noMaximum)

	def getValues(cls, parent):
		"shows the dialog box and returns maximum number of solutions input by the user"
		if not cls.dialog: cls.dialog = MaxDialog(parent)
		startVal = cls.dialog.max and str(cls.dialog.max) or ""
		cls.dialog.maxField.SetValue(startVal)
		cls.dialog.ShowModal()
		if cls.dialog.cancelled: return (False, None)
		else: return (True, cls.dialog.max)
	getValues = classmethod(getValues)

	def __ok(self, event):
		self.cancelled = False
		try:
			value = int(str(self.maxField.GetValue()))
		except ValueError: return
		if value < 2: return
		self.max = value
		self.Hide()
	
	def __cancel(self, event):
		self.cancelled = True
		self.Hide()

	def __noMaximum(self, event):
		self.cancelled = False
		self.max = 0
		self.Hide()

#*********************** class AboutDlg *******************************

class AboutDlg(wx.Dialog):
	
	def __init__(self, parent):

		wx.Dialog.__init__(self, parent, -1, "about Sudoku")
		self.bitmap = wx.BitmapFromImage(wx.Image("image/splash.png"))
		self.SetClientSize(self.bitmap.GetSize())
		wx.EVT_PAINT(self,self.paint)

	def paint(self, event):
		paintDC = wx.PaintDC(self)
		paintDC.DrawBitmap(self.bitmap,0,0)
		paintDC.DrawText('version %s' % VERSION, 10, 10)


#*********************** class GuiApp ********************************

class GuiApp(wx.App):
	
	def __init__(self, list, windowClass=SudokuWindow, *args, **kwds):
		self.options = list[0]
		self.args = list[1]
		self.windowClass = windowClass
		wx.App.__init__(self, *args, **kwds)
	
	def OnInit(self):
		if self.options.xmlin:
			fileData = {'xmlFile': self.options.inputFile}
		elif self.options.csvin:
			fileData = {'csvFile': self.options.inputFile}
		else:
			fileData = {'file': self.options.inputFile}

		g = sudokuH.HGrid(size=eval(self.options.size), **fileData)
		frame = self.windowClass(grid=g,app=self)
		self.SetTopWindow(frame)
		self.SetExitOnFrameDelete(True)
		frame.Show(1)
		self.MainLoop()
		return True
		
#****************************** main program *********************************

def parseOptions():
	from optparse import OptionParser
	parser = OptionParser()
	parser.add_option("-i", "--input", 
						action="store", 
						dest="inputFile", 
						default="BLANK", 
						help="""Read initial state from INPUTFILE, 
								or if INPUTFILE = - from sys.input 
								(no prompts will be given).
								If this option is not set a size must be specified.""")
	parser.add_option("-x", "--xml-input", 
						action="store_true", 
						dest="xmlin", 
						default=False,
						help="""Indicates that the input
						file is in xml format.
						Only useful in conjunction with -i""")
	
	parser.add_option("-c", "--csv-input", 
						action="store_true", 
						dest="csvin", 
						default = False, 
						help="""Stipulates that the input file is in 
						'comma separated value' format
						Only useful in conjunction with -i""")
		
	parser.add_option("-z", "--grid-size", 
						action="store", 
						dest="size", 
						default = "3", 
						help="""Size of sub-grid. May be 2, 3, or 4 (default 3).
								If an input file is specified this option will 
								be ignored.""")

	return parser.parse_args()

def main(*args, **kwds):
	(options, args) = parseOptions()
	
	GuiApp((options, args))
	return

if __name__=="__main__":
	main(sys.argv)
