import wx, wx.grid, wx.html, sys
from sudoSkeleton import Skeleton
import sudoku

#****************************** class Table *********************************

class Table(wx.grid.Grid):
	"the control that draws the grid on the screen"

	def __init__(self, parent, size=3, grid=None, cellSize=32):
		self.grid = grid
		if self.grid: self.__size = grid.size
		else: self.__size = size
		wx.grid.Grid.__init__(self,parent,-1,size=(cellSize*size**2+20,cellSize*size**2+20))
		self.SetColLabelSize(0)
		self.SetRowLabelSize(0)
		self.SetDefaultColSize(cellSize)
		self.SetDefaultRowSize(cellSize)
		self.SetGridLineColour(wx.Color(150,150,150))
		self.CreateGrid(self.__size**2, self.__size**2)
		for row in range(self.__size*self.__size):
			for col in range(self.__size*self.__size):
				if (row // self.__size + col // self.__size) % 2:
					self.SetCellBackgroundColour(row,col,wx.Color(255,200,200))
				else:
					self.SetCellBackgroundColour(row,col,wx.Color(200,255,200))
		self.SetDefaultCellAlignment(wx.ALIGN_CENTRE,wx.ALIGN_CENTRE)
		self.SetDefaultCellFont(wx.Font(family=wx.SWISS, style=wx.NORMAL, weight=wx.NORMAL, pointSize=16, faceName="clean"))
		self.parent = parent
		self._setGrid()
		self.EnableEditing(False)
		self.EnableDragGridSize(False)
		self.undoBuffer = UndoBuffer(self, lambda: None, None)
		self.permitSelection = False

		wx.grid.EVT_GRID_CELL_LEFT_CLICK(self, self.leftClicked)
		wx.grid.EVT_GRID_CELL_RIGHT_CLICK(self, self.rightClicked)
		wx.grid.EVT_GRID_RANGE_SELECT(self, self.onSelect)
		wx.grid.EVT_GRID_SELECT_CELL(self, self.onSelect)
	
	def _setGrid(self): self.grid = Skeleton()

	def __dontSelect(self, event):
		"remove selection immediately"
		print "in dont select"
		if event.Selecting():
			self.ClearSelection()
			event.Skip()

	def __passit(self, event): pass

	def onSelect(self, event):
		if self.permitSelection: pass
		else:
			#remove selection immediately
			if event.Selecting():
				self.ClearSelection()
				event.Skip()

	def select(self, on):
		self.permitSelection = on
	
	def __getSize(self): return self.__size
	size = property(__getSize)

	def readAllValues(self, showingSteps=False):
		"reads values from the underlying grid into the display"

		for i in range(self.size**2):
			for j in range(self.size**2):
				self.SetCellValue(i, j, self.grid.getValue(i,j) and "#" or "")

	def clearAllValues(self):
		"clears all values from the screen and the underlying grid"
		self.grid.zeroValues()
		for i in range(self.size**2):
			for j in range(self.size**2):
				self.SetCellValue("")

	def open(self, file=None, string=None):
		"reads data from a file or string - specify only one"
		if string: self.grid.setReader(string=string)
		else: self.grid.setReader(file=file)
		if self.grid.size <> self.size:
			self.parent.SetStatusText("sorry only %d by %d grids will fit in this window - (you could start another)" % (self.size*self.size, self.size*self.size))
			return
		self.grid.zeroValues()
		self.grid.read()
		self.readAllValues()
		#self.undoBuffer.store()
		
	def __clicked(self, event, set):
		"sets or unsets the clicked cell"
		row = event.GetRow()
		col = event.GetCol()
		self.grid.set(r=row, c=col, value=set)
		self.readAllValues()
		self.parent.SetStatusText(str(self.grid.count()),self.parent.counter)
		
	def rightClicked(self, event):
		self.__clicked(event, set=False)
		
	def leftClicked(self, event):
		self.__clicked(event, set=True)

#****************************** class Window *********************************

class Window(wx.Frame):

	def __init__(self, app=None, size=3):
		self.app = app
		wx.Frame.__init__(self, None, -1, "sudoku skeleton", size=(325,375))
		self.clearClickedItem = 0
		self.restoreItem = 0
		self.table = Table(self,size)
		self.CreateStatusBar(2)
		self.SetStatusWidths([-1,35])
		self.counter = 1
		self.printer = None
		
		fm = wx.Menu()
		mOpen = fm.Append(-1, "&open...\tCTRL+O").GetId() #self.open 
		mSave = fm.Append(-1, "&save as...\tCTRL+S").GetId() #self.save 
		fm.AppendSeparator()
		mPrint = fm.Append(-1, "&print...\tCTRL+P").GetId()
		fm.AppendSeparator()
		mExit = fm.Append(wx.ID_EXIT, "e&xit\tCTRL+X").GetId() #self.exit 
		mb = wx.MenuBar()
		mb.Append(fm,"&file")

		em = wx.Menu()
		sm = wx.Menu()
		symmetryTypes = ( "&s shaped", 
							"&kaleidoscope", 
							"s&quare", 
							"&pinwheel", 
							"&bi-symmetry",
							"&diagonal",
							"&none" )
		self.symmetryTypes = {}
		for s in symmetryTypes:
			ms = sm.Append(-1, s).GetId() #self.setSymmetry
			# set the symmetry type to the letter after the & 
			# in the menu text, converted to uppercase
			self.symmetryTypes[ms] = s[s.find('&')+1].upper()
		em.AppendMenu(-1, "s&ymmetry", sm)
		self.mUndo = em.Append(-1, "&undo\tCTRL+Z").GetId() #self.undo 
		self.mRedo = em.Append(-1, "&redo\tCTRL+R").GetId() #self.redo 
		mFont = em.Append(-1, "fo&nt\tCTRL+N").GetId() #self.font 
		mb.Append(em,"&edit")
		self.SetMenuBar(mb)
		wx.EVT_MENU(self, mOpen, self.open)
		wx.EVT_MENU(self, mSave, self.save)
		wx.EVT_MENU(self, mExit, self.exit)
		wx.EVT_MENU(self, mFont, self.font)
		wx.EVT_MENU(self, mPrint, self.printIt)
		for i in self.symmetryTypes.keys():
			wx.EVT_MENU(self, i, self.setSymmetry)

	def printIt(self, event):
		if not self.printer: self.printer = Printer(self)
		self.printer.printit(self.table.grid)
	
	def exit(self, event):
		self.Close()
		self.app.ExitMainLoop() # this is needed if dbDialog is used!!

	def font(self, event):
		dialog = wx.FontDialog(self,wx.FontData())
		ok = dialog.ShowModal() == wx.ID_OK
		if not ok: return
		font = dialog.GetFontData().GetChosenFont()
		self.table.SetDefaultCellFont(font)

	def open(self, event):
		dialog = wx.FileDialog(self,"file to open","","","All files|*|Skeleton files (*.skel)|*.skel|Data files (*.dat)|*.dat",
				wx.OPEN|wx.FILE_MUST_EXIST)
		ok = dialog.ShowModal() == wx.ID_OK
		name = dialog.GetPath()
		if not ok: return
		self.table.open(file=name)
		self.SetStatusText("read data from file: %s" % name)
		self.SetTitle("sudoku skeleton - %s" % name)
		self.SetStatusText(str(self.table.grid.count()),self.counter)

	def save(self, event):
		dialog = wx.FileDialog(self,"file to open","","","",
				wx.SAVE|wx.OVERWRITE_PROMPT)
		ok = dialog.ShowModal() == wx.ID_OK
		if not ok: return
		name = dialog.GetPath()
		file = open(name, "w")
		file.write(self.table.grid.writeIt())
		self.SetStatusText("grid saved to file: %s" % name)
		self.SetTitle("sudoku skeleton - %s" % name)

	def setSymmetry(self, event): 
		self.table.grid.symmetry = self.symmetryTypes[event.GetId()]

#************************** class Printer *******************************

class Printer(wx.Printout):

	def __init__(self, parent):
		self.parent = parent
		wx.Printout.__init__(self)
		self.grid = None
		(self.marginx, self.marginy) = 50, 80
		(self.lengthx, self.lengthy) = 500, 500

	def printit(self, grid):
		printer = wx.Printer()
		self.grid = grid
		printer.Print(self.parent, self, True)

	def OnPrintPage(self, n):
		n=self.grid.size
		self.drawBlank()
	
		self.GetDC().SetFont(wx.Font(30, wx.FONTFAMILY_ROMAN, 
				wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
		(stepx, stepy) = self.lengthx/n/n, self.lengthy/n/n
		startx = self.marginx + self.lengthx/n/n/2
		starty = self.marginy + self.lengthy/n/n/2
		for x in range(n*n):
			for y in range(n*n):
				self.centreText(self.grid.listValues()[y*n*n+x] and '*' or '', 
						startx+x*stepx, starty+y*stepy) 
	
	def drawBlank(self):
		dc = self.GetDC()
		n=self.grid.size
		(stepx, stepy) = self.lengthx/n/n, self.lengthy/n/n
		thickPen = wx.Pen('black', 10, wx.SOLID)
		thinPen = wx.Pen('grey', 1, wx.SOLID)
		dc.SetPen(thinPen)
		for y1 in range(self.marginy, self.marginy+self.lengthy, stepy):
			dc.DrawLine(self.marginx, y1, self.marginx+self.lengthx, y1)
		for x1 in range(self.marginx, self.marginx+self.lengthx, stepx):
			dc.DrawLine(x1, self.marginy, x1, self.marginy+self.lengthy)
		dc.SetPen(thickPen)
		for y1 in range(self.marginy + stepy*n, self.marginy+self.lengthy-stepy*n, stepy*n):
			dc.DrawLine(self.marginx+self.lengthx, y1, self.marginx, y1)
		for x1 in range(self.marginx+stepx*n, self.marginx+self.lengthx-stepx*n, stepx*n):
			dc.DrawLine(x1, self.marginy, x1, self.marginy+self.lengthy)
		dc.SetBrush(wx.TRANSPARENT_BRUSH)
		dc.DrawRectangle(self.marginx, self.marginy, self.lengthx, self.lengthy)
		
	def centreText(self, text, x, y):
		if not text: return
		dc = self.GetDC()
		(w,h) = dc.GetTextExtent(text)
		dc.DrawText(text, x-w/2, y-h/2)

#****************************** class UndoBuffer *********************************

class UndoBuffer(object):

	def __init__(self, parent, storefunc, restorefunc):
		self.parent = parent
		self.pointer = 0
		self.buffer = []
		self.storefunc = storefunc
		self.restorefunc = restorefunc
		self.store()

	def store(self):
		self.pointer += 1
		if len(self.buffer) > self.pointer:
			del self.buffer[self.pointer:]
		self.buffer.append(self.storefunc())
		self.pointer = len(self.buffer) - 1

	def restore(self):
		if self.pointer > -1 and self.pointer < len(self.buffer):
			self.restorefunc(self.buffer[self.pointer])

	def undo(self):
		if self.pointer > 0:
			self.pointer -= 1
			self.restore()

	def redo(self):
		if self.pointer < len(self.buffer) - 1:
			self.pointer += 1
			self.restore()

#*********************** class SkeletonGuiApp ********************************

class SkeletonGuiApp(wx.App):
	
	def __init__(self, list, *args, **kwds):
		self.options = list[0]
		self.args = list[1]
		wx.App.__init__(self, *args, **kwds)
	
	def OnInit(self):
		frame = Window(app=self)
		self.SetTopWindow(frame)
		self.SetExitOnFrameDelete(True)
		frame.Show(1)
		self.MainLoop()
		return True
		
#****************************** main program *********************************

def main(*args, **kwds):
	from optparse import OptionParser
	parser = OptionParser()
	parser.add_option("-i", "--input", 
						action="store", 
						dest="inputFile", 
						default="BLANK", 
						help="read initial state from INPUTFILE, or if INPUTFILE = - from sys.input (no prompts will be given).  If this option is not set a size must be specified")
	parser.add_option("-z", "--grid size", 
						action="store", 
						dest="size", 
						default = "3", 
						help="size of sub-grid may be 2, 3, or 4 if an input file is specified this option will be ignored")
	(options, args) = parser.parse_args()
	
	SkeletonGuiApp((options, args))
	return

if __name__=="__main__":
	main(sys.argv)
