import xml.dom.minidom, math

class XmlWriter(object):

	def __init__(self, grid, possibles=None):
		self.dom = xml.dom.minidom.getDOMImplementation().createDocument(
														None, "sudoku", None)
		top = self.dom.documentElement
		size = self.dom.createElement("size")
		s = str(grid.size**2)
		for at in ("rows", "cols"):
			attribute = self.dom.createAttribute(at)
			attribute.value = s
			size.setAttributeNode(attribute)
		top.appendChild(size)
		for cell in grid.cells:
			if cell.value: top.appendChild(self.__writeCell(cell))
			elif possibles and possibles[cell.number]:
				el = self.__writeCell(cell)
				for p in possibles[cell.number]:
					el.appendChild(self.__writeCandidate(p))
				top.appendChild(el)
			
	def __writeCandidate(self, v):
		element = self.dom.createElement("candidate")
		attribute = self.dom.createAttribute("value")
		attribute.value = str(v)
		element.setAttributeNode(attribute)
		return element

	def __writeCell(self, cell):
		element = self.dom.createElement("cell")
		for attribute in ("row", "col", "value"):
			self.__addCellAttr(cell, attribute, element)
		return element

	def __addCellAttr(self, cell, name, element):
		attribute = self.dom.createAttribute(name)
		v = eval("cell."+name)
		# convert row and column numbers from 0 based to 1 based
		if name in ("row", "col"): attribute.value = str(v+1)
		else: attribute.value = str(v)
		element.setAttributeNode(attribute)

	def __str__(self):
		return self.dom.toxml()

class XmlReader(object):

	def __init__(self, file):
		self.__size = 0
		self.grid = None
		self.__possibles = []
		self.dom = xml.dom.minidom.parse(file)
		self.readSize(self.dom.getElementsByTagName("size")[0])
	
	def __getSize(self): return self.__size
	size = property(__getSize)

	def __getPossibles(self): return self.__possibles
	possibles = property(__getPossibles)
	
	def readSize(self, siz):
		try:
			rows = int(siz.getAttribute("rows"))
			cols = int(siz.getAttribute("cols"))
		except ValueError:
			raise XmlParseError("size element not correctly specified")
		if rows != cols: 
			raise XmlParseError("number of rows must equal number of columns")
		self.__size = int(math.sqrt(rows))
		if self.size**2 != rows: 
			raise XmlParseError("grid size must be a square number")
		self.__possibles = []
		for i in range(self.size**4): self.__possibles.append([])

	def readCells(self, grid):
		self.grid = grid
		for cell in self.dom.getElementsByTagName("cell"): 
			self.readCell(cell)
		return grid
	
	def readCell(self, cell):
		try:
			row = int(cell.getAttribute("row"))
			col = int(cell.getAttribute("col"))
		except ValueError:
			raise XmlParseError("invalid row or col value for cell")
		try:
			v = cell.getAttribute("value")
			value = int(v)
			self.grid.setValue(row-1, col-1, value, update=False)
			if value == 0: self.readCandidates(cell, row, col)
		except ValueError:
			if v == "": self.readCandidates(cell, row, col)
			else: 
				raise XmlParseError("invalid value for cell (%d,%d)" % (row, col))
		
	def readCandidates(self, cell, row, col):
		possibles = []
		for candidate in cell.getElementsByTagName("candidate"):
			possibles.append(self.readCandidate(candidate))
		self.__possibles[(row-1)*self.size**2 + col-1] = possibles[:]

	def readCandidate(self, candidate):
		try:
			return int(candidate.getAttribute("value"))
		except ValueError:
			raise XmlParseError("invalid candidate value for cell")

	def close(self): self.dom.unlink()


#**************************** class XmlParseError **************************"

class XmlParseError(Exception):

        def __init__(self, value):
                self.value = value

        def __str__(self):
                return repr(self.value)

